@@ -30,6 +30,7 @@ pub(super) struct ProgressBarState {
     hidden_no_capture: bool,
     hidden_run_paused: bool,
     hidden_info_response: bool,
+    hidden_between_sub_runs: bool,
 }
 
 impl ProgressBarState {
@@ -63,6 +64,7 @@ impl ProgressBarState {
             hidden_no_capture: false,
             hidden_run_paused: false,
             hidden_info_response: false,
+            hidden_between_sub_runs: false,
         }
     }
 
@@ -70,17 +72,24 @@ impl ProgressBarState {
         let before_should_hide = self.should_hide();
 
         match &event.kind {
+            TestEventKind::StressSubRunFinished { .. } => {
+                // Hide the progress bar between sub runs to avoid a spurious
+                // progress bar.
+                self.hidden_between_sub_runs = true;
+            }
             TestEventKind::SetupScriptStarted { no_capture, .. } => {
                 // Hide the progress bar if either stderr or stdout are being passed through.
                 if *no_capture {
                     self.hidden_no_capture = true;
                 }
+                self.hidden_between_sub_runs = false;
             }
             TestEventKind::SetupScriptFinished { no_capture, .. } => {
                 // Restore the progress bar if it was hidden.
                 if *no_capture {
                     self.hidden_no_capture = false;
                 }
+                self.hidden_between_sub_runs = false;
             }
             TestEventKind::TestStarted {
                 current_stats,
@@ -94,6 +103,7 @@ impl ProgressBarState {
                 cancel_state,
                 ..
             } => {
+                self.hidden_between_sub_runs = false;
                 self.bar
                     .set_prefix(progress_bar_prefix(current_stats, *cancel_state, styles));
                 self.bar
@@ -165,7 +175,10 @@ impl ProgressBarState {
     }
 
     fn should_hide(&self) -> bool {
-        self.hidden_no_capture || self.hidden_run_paused || self.hidden_info_response
+        self.hidden_no_capture
+            || self.hidden_run_paused
+            || self.hidden_info_response
+            || self.hidden_between_sub_runs
     }
 }
 
@@ -236,6 +249,8 @@ impl TerminalProgress {
     ) -> Result<(), io::Error> {
         let value = match &event.kind {
             TestEventKind::RunStarted { .. }
+            | TestEventKind::StressSubRunStarted { .. }
+            | TestEventKind::StressSubRunFinished { .. }
             | TestEventKind::SetupScriptStarted { .. }
             | TestEventKind::SetupScriptSlow { .. }
             | TestEventKind::SetupScriptFinished { .. } => TerminalProgressValue::None,
