@@ -26,7 +26,7 @@ use crate::{
     config::elements::LeakTimeoutResult,
     errors::{DisplayErrorChain, FormatVersionError, FormatVersionErrorInner, WriteEventError},
     list::RustTestSuite,
-    reporter::events::{ExecutionResult, TestEvent, TestEventKind},
+    reporter::events::{ExecutionResult, StressIndex, TestEvent, TestEventKind},
     test_output::{ChildExecutionOutput, ChildOutput, ChildSingleOutput},
 };
 use bstr::ByteSlice;
@@ -82,6 +82,8 @@ struct LibtestSuite<'cfg> {
     filtered: usize,
     /// The number of tests in this suite that are still running
     running: usize,
+
+    stress_index: Option<StressIndex>,
     meta: &'cfg RustTestSuite<'cfg>,
     /// The accumulated duration of every test that has been executed
     total: std::time::Duration,
@@ -222,20 +224,24 @@ impl<'cfg> LibtestReporter<'cfg> {
         let mut retries = None;
 
         // Write the pieces of data that are the same across all events
-        let (kind, eve, test_instance) = match &event.kind {
-            TestEventKind::TestStarted { test_instance, .. } => {
-                (KIND_TEST, EVENT_STARTED, test_instance)
-            }
+        let (kind, eve, stress_index, test_instance) = match &event.kind {
+            TestEventKind::TestStarted {
+                stress_index,
+                test_instance,
+                ..
+            } => (KIND_TEST, EVENT_STARTED, stress_index, test_instance),
             TestEventKind::TestSkipped {
+                stress_index,
                 test_instance,
                 reason: MismatchReason::Ignored,
             } => {
                 // Note: unfortunately, libtest does not expose the message test in `#[ignore = "<message>"]`
                 // so we can't replicate the behavior of libtest exactly by emitting
                 // that message as additional metadata
-                (KIND_TEST, EVENT_STARTED, test_instance)
+                (KIND_TEST, EVENT_STARTED, stress_index, test_instance)
             }
             TestEventKind::TestFinished {
+                stress_index,
                 test_instance,
                 run_statuses,
                 ..
@@ -258,10 +264,11 @@ impl<'cfg> LibtestReporter<'cfg> {
                         | ExecutionResult::ExecFail
                         | ExecutionResult::Timeout => EVENT_FAILED,
                     },
+                    stress_index,
                     test_instance,
                 )
             }
-            TestEventKind::RunFinished { .. } => {
+            TestEventKind::StressSubRunFinished { .. } | TestEventKind::RunFinished { .. } => {
                 for test_suite in std::mem::take(&mut self.test_suites).into_values() {
                     self.finalize(test_suite)?;
                 }
@@ -299,11 +306,21 @@ impl<'cfg> LibtestReporter<'cfg> {
 
                 if self.emit_nextest_obj {
                     write!(
-                        &mut out,
-                        r#","nextest":{{"crate":"{crate_name}","test_binary":"{binary_name}","kind":"{}"}}"#,
+                        out,
+                        r#","nextest":{{"crate":"{crate_name}","test_binary":"{binary_name}","kind":"{}""#,
                         suite_info.kind,
                     )
                     .map_err(fmt_err)?;
+
+                    if let Some(stress_index) = stress_index {
+                        write!(out, r#","stress_index":{}"#, stress_index.current)
+                            .map_err(fmt_err)?;
+                        if let Some(total) = stress_index.total {
+                            write!(out, r#","stress_total":{total}"#).map_err(fmt_err)?;
+                        }
+                    }
+
+                    write!(out, "}}").map_err(fmt_err)?;
                 }
 
                 out.extend_from_slice(b"}\n");
@@ -314,6 +331,7 @@ impl<'cfg> LibtestReporter<'cfg> {
                     succeeded: 0,
                     ignored,
                     filtered,
+                    stress_index: *stress_index,
                     meta: test_instance.suite_info,
                     total: std::time::Duration::new(0, 0),
                     ignore_block: None,
@@ -343,13 +361,16 @@ impl<'cfg> LibtestReporter<'cfg> {
         // as libtest does not support that functionality
         write!(
             out,
-            r#"{{"type":"{kind}","event":"{eve}","name":"{}::{}${}"#,
+            r#"{{"type":"{kind}","event":"{eve}","name":"{}::{}"#,
             suite_info.package.name(),
             suite_info.binary_name,
-            test_instance.name,
         )
         .map_err(fmt_err)?;
 
+        if let Some(stress_index) = stress_index {
+            write!(out, "@stress-{}", stress_index.current).map_err(fmt_err)?;
+        }
+        write!(out, "${}", test_instance.name).map_err(fmt_err)?;
         if let Some(retry_count) = retries {
             write!(out, "#{retry_count}\"").map_err(fmt_err)?;
         } else {
@@ -423,14 +444,31 @@ impl<'cfg> LibtestReporter<'cfg> {
 
         out.extend_from_slice(b"}\n");
 
-        // If this is the last test of the suite, emit the test suite summary
-        // before emitting the entire block
-        if test_suite.running > 0 {
-            return Ok(());
-        }
+        if self.emit_nextest_obj {
+            {
+                use std::io::Write as _;
 
-        if let Some(test_suite) = self.test_suites.remove(suite_info.binary_id.as_str()) {
-            self.finalize(test_suite)?;
+                let mut stdout = std::io::stdout().lock();
+                stdout.write_all(out).map_err(WriteEventError::Io)?;
+                stdout.flush().map_err(WriteEventError::Io)?;
+                out.clear();
+            }
+
+            if test_suite.running == 0 {
+                if let Some(test_suite) = self.test_suites.remove(suite_info.binary_id.as_str()) {
+                    self.finalize(test_suite)?;
+                }
+            }
+        } else {
+            // If this is the last test of the suite, emit the test suite summary
+            // before emitting the entire block
+            if test_suite.running > 0 {
+                return Ok(());
+            }
+
+            if let Some(test_suite) = self.test_suites.remove(suite_info.binary_id.as_str()) {
+                self.finalize(test_suite)?;
+            }
         }
 
         Ok(())
@@ -469,10 +507,19 @@ impl<'cfg> LibtestReporter<'cfg> {
             let binary_name = &suite_info.binary_name;
             write!(
                 out,
-                r#","nextest":{{"crate":"{crate_name}","test_binary":"{binary_name}","kind":"{}"}}"#,
+                r#","nextest":{{"crate":"{crate_name}","test_binary":"{binary_name}","kind":"{}""#,
                 suite_info.kind,
             )
             .map_err(fmt_err)?;
+
+            if let Some(stress_index) = test_suite.stress_index {
+                write!(out, r#","stress_index":{}"#, stress_index.current).map_err(fmt_err)?;
+                if let Some(total) = stress_index.total {
+                    write!(out, r#","stress_total":{total}"#).map_err(fmt_err)?;
+                }
+            }
+
+            write!(out, "}}").map_err(fmt_err)?;
         }
 
         out.extend_from_slice(b"}\n");
